/*
 * ParametricModuleControlPanel.java
 *
 * Created on August 2, 2006, 2:57 PM
 */

package sddm.modules;

import sddm.*;

/**
 *
 * @author  Erik J. Tollerud
 */
public class ParametricModuleControlPanel extends SDDModuleControlPanel {
    private ParametricModule mod;
    
    /** Creates new form ParametricModuleControlPanel */
    public ParametricModuleControlPanel(ParametricModule module) {
        this.mod = module;
        
        initComponents();
        
        if (mod.getSystem().getGen().initialXs().length == 1) {
            this.x0Field.setEnabled(true);
            this.y0Field.setEnabled(true);
            this.z0Field.setEnabled(true);
            this.w0Field.setEnabled(true);
            this.minRangeField.setEnabled(true);
            this.maxRangeField.setEnabled(true);
            this.rangeButton.setEnabled(true);
        } else {
            this.x0Field.setEnabled(false);
            this.y0Field.setEnabled(false);
            this.z0Field.setEnabled(false);
            this.w0Field.setEnabled(false);
            this.minRangeField.setEnabled(false);
            this.maxRangeField.setEnabled(false);
            this.rangeButton.setEnabled(false);
        }
        
        
    }

    public SDDModule panelToModuleSettings(SDDModule module) {
        ParametricModule pm = (ParametricModule)module;
        pm.setDrawingSize(((Number)this.lineWidthField.getValue()).floatValue());
        boolean ird = this.iterOnReDispCheckBox.isSelected();
        if (ird) {
            pm.setIterateOnRedisplay(pm.getIterateOnRedisplay());
        }
        else
            pm.setIterateOnRedisplay(0);
        
        return module;
    }
    
    public void moduleSettingsToPanel(SDDModule module) {
        ParametricModule pm = (ParametricModule)module;
        if (pm.getSystem().getGen().initialXs().length == 1) {
            this.x0Field.setEnabled(true);
            this.y0Field.setEnabled(true);
            this.z0Field.setEnabled(true);
            this.w0Field.setEnabled(true);
            this.minRangeField.setEnabled(true);
            this.maxRangeField.setEnabled(true);
            this.rangeButton.setEnabled(true);
            
            this.x0Field.setValue(pm.getSystem().getGen().initialXs()[0]);
            this.y0Field.setValue(pm.getSystem().getGen().initialYs()[0]);
            this.z0Field.setValue(pm.getSystem().getGen().initialZs()[0]);
            this.w0Field.setValue(pm.getSystem().getGen().initialWs()[0]);
            this.minRangeField.setValue(pm.getSystem().getGen().initialTs()[0]);
        } else {
            this.x0Field.setEnabled(false);
            this.y0Field.setEnabled(false);
            this.z0Field.setEnabled(false);
            this.w0Field.setEnabled(false);
            this.minRangeField.setEnabled(false);
            this.maxRangeField.setEnabled(false);
            this.rangeButton.setEnabled(false);
        }
        this.lineWidthField.setValue(pm.getDrawingSize());
        
        this.currIterField.setText(Integer.toString(mod.getSystem().getCurrentIteration()));
        
        int ior = pm.getIterateOnRedisplay();
        if (ior > 0) {
            this.iterOnReDispCheckBox.setSelected(true);
            this.iterationsField.setValue(ior);
        }
        else
            this.iterOnReDispCheckBox.setSelected(false);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        iterButton = new javax.swing.JButton();
        rangeButton = new javax.swing.JButton();
        minRangeField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        maxRangeField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        iterationsField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        x0Field = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        y0Field = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        z0Field = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        w0Field = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        jLabel1 = new javax.swing.JLabel();
        newSysButton = new javax.swing.JButton();
        bgColorButton = new javax.swing.JButton();
        colorSchemeButton = new javax.swing.JButton();
        jLabel2 = new javax.swing.JLabel();
        lineWidthField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        currIterField = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        targetCenterButton = new javax.swing.JButton();
        iterOnReDispCheckBox = new javax.swing.JCheckBox();
        jLabel4 = new javax.swing.JLabel();
        granField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getIntegerInstance());
        saveButton = new javax.swing.JButton();
        joinCheckBox = new javax.swing.JCheckBox();

        setPreferredSize(new java.awt.Dimension(454, 159));
        iterButton.setText("Iterate");
        iterButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                iterButtonActionPerformed(evt);
            }
        });

        rangeButton.setText("Generate Range");
        rangeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                rangeButtonActionPerformed(evt);
            }
        });

        minRangeField.setText("num");
        minRangeField.setValue(0.0f);
        minRangeField.setPreferredSize(new java.awt.Dimension(minRangeField.getPreferredSize().width+15,minRangeField.getPreferredSize().height));

        maxRangeField.setText("num");
        maxRangeField.setValue(1.0f);
        maxRangeField.setPreferredSize(new java.awt.Dimension(maxRangeField.getPreferredSize().width+15,maxRangeField.getPreferredSize().height));

        iterationsField.setText("num");
        iterationsField.setPreferredSize(new java.awt.Dimension(45, 21));
        iterationsField.setValue(1);
        iterationsField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                iterationsFieldActionPerformed(evt);
            }
        });

        x0Field.setText("num");
        x0Field.setValue(mod.getSystem().getGen().initialXs()[0]);
        x0Field.setPreferredSize(new java.awt.Dimension(x0Field.getPreferredSize().width+15,x0Field.getPreferredSize().height));

        y0Field.setText("num");
        y0Field.setValue(mod.getSystem().getGen().initialYs()[0]);
        y0Field.setPreferredSize(new java.awt.Dimension(y0Field.getPreferredSize().width+15,y0Field.getPreferredSize().height));

        z0Field.setText("num");
        z0Field.setValue(mod.getSystem().getGen().initialZs()[0]);
        z0Field.setPreferredSize(new java.awt.Dimension(z0Field.getPreferredSize().width+15,z0Field.getPreferredSize().height));

        w0Field.setText("num");
        w0Field.setValue(mod.getSystem().getGen().initialWs()[0]);
        w0Field.setPreferredSize(new java.awt.Dimension(w0Field.getPreferredSize().width+15,w0Field.getPreferredSize().height));

        jLabel1.setText("ICs for Range");

        newSysButton.setText("New System...");
        newSysButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newSysButtonActionPerformed(evt);
            }
        });

        bgColorButton.setText("Background Color...");
        bgColorButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bgColorButtonActionPerformed(evt);
            }
        });

        colorSchemeButton.setText("Color Scheme...");
        colorSchemeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                colorSchemeButtonActionPerformed(evt);
            }
        });

        jLabel2.setText("Line/Point Width");

        lineWidthField.setText("num");
        lineWidthField.setValue(mod.getDrawingSize());
        lineWidthField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lineWidthFieldActionPerformed(evt);
            }
        });

        currIterField.setEditable(false);
        currIterField.setText(Integer.toString(mod.getSystem().getCurrentIteration()));

        jLabel3.setText("Iterations:");

        targetCenterButton.setText("Target Center of Data");
        targetCenterButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetCenterButtonActionPerformed(evt);
            }
        });

        iterOnReDispCheckBox.setText("Iterate on Redisplay");
        iterOnReDispCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        iterOnReDispCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        iterOnReDispCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                iterOnReDispCheckBoxActionPerformed(evt);
            }
        });

        jLabel4.setText("Granularity");

        granField.setText("num");
        granField.setValue(mod.getGranularity());
        granField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                granFieldActionPerformed(evt);
            }
        });

        saveButton.setText("Save Data...");
        saveButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveButtonActionPerformed(evt);
            }
        });

        joinCheckBox.setSelected(mod.isJoined());
        joinCheckBox.setText("join points?");
        joinCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        joinCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        joinCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                joinCheckBoxActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(layout.createSequentialGroup()
                                .add(rangeButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(minRangeField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(maxRangeField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(layout.createSequentialGroup()
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(layout.createSequentialGroup()
                                        .add(iterButton)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(iterationsField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                    .add(newSysButton))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(currIterField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 71, Short.MAX_VALUE)
                                    .add(jLabel3))))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(layout.createSequentialGroup()
                                .add(jLabel1)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(x0Field, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(y0Field, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(z0Field, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(w0Field, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(layout.createSequentialGroup()
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(colorSchemeButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(iterOnReDispCheckBox))
                            .add(bgColorButton)))
                    .add(layout.createSequentialGroup()
                        .add(jLabel2)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(lineWidthField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jLabel4)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(granField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(joinCheckBox))
                    .add(layout.createSequentialGroup()
                        .add(targetCenterButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(saveButton)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(newSysButton)
                        .add(colorSchemeButton))
                    .add(jLabel3)
                    .add(iterOnReDispCheckBox))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(iterButton)
                    .add(iterationsField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(bgColorButton)
                    .add(currIterField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(rangeButton)
                    .add(minRangeField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(maxRangeField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel1)
                    .add(x0Field, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(y0Field, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(z0Field, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(w0Field, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2)
                    .add(lineWidthField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel4)
                    .add(granField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(joinCheckBox))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(targetCenterButton)
                    .add(saveButton))
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void joinCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_joinCheckBoxActionPerformed
        mod.setJoined(joinCheckBox.isSelected());
        this.redisplayParent();
    }//GEN-LAST:event_joinCheckBoxActionPerformed

    private void saveButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveButtonActionPerformed
        javax.swing.JFileChooser jfc = new javax.swing.JFileChooser();
        jfc.setMultiSelectionEnabled(false);
        jfc.setDialogTitle("File for Saving Parametric Data");
        if(jfc.showSaveDialog(this)==jfc.APPROVE_OPTION) {
            boolean saveW = (javax.swing.JOptionPane.showConfirmDialog(this,"Save W Component?","W Component?",javax.swing.JOptionPane.YES_NO_OPTION)==javax.swing.JOptionPane.YES_OPTION);
            try {
            this.mod.savePoints(jfc.getSelectedFile(),saveW);
            }
            catch (java.io.IOException e) {
                javax.swing.JOptionPane.showMessageDialog(null,"I/O Problem:"+e,"Couldn't Save File",javax.swing.JOptionPane.ERROR_MESSAGE);
            }
        }
        
        
    }//GEN-LAST:event_saveButtonActionPerformed

    private void granFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_granFieldActionPerformed
        mod.setGranularity(((Number)granField.getValue()).intValue());
        
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_granFieldActionPerformed

    private void iterationsFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_iterationsFieldActionPerformed
        iterOnReDispCheckBoxActionPerformed(evt);
        mod.iterPerKey = ((Number)this.iterationsField.getValue()).intValue();
    }//GEN-LAST:event_iterationsFieldActionPerformed

    private void iterOnReDispCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_iterOnReDispCheckBoxActionPerformed
        if (iterOnReDispCheckBox.isSelected())
            mod.setIterateOnRedisplay(((Number)this.iterationsField.getValue()).intValue());
        else
            mod.setIterateOnRedisplay(0);
    }//GEN-LAST:event_iterOnReDispCheckBoxActionPerformed

    private void targetCenterButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetCenterButtonActionPerformed
        sddm.data.PointList pl = this.mod.getSystem().getPointList();
        this.cam.setTargetPoint(pl.getXAvg(),pl.getYAvg(),pl.getZAvg());
        
        this.redisplayParent();
    }//GEN-LAST:event_targetCenterButtonActionPerformed
    
    private void lineWidthFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lineWidthFieldActionPerformed
        mod.setDrawingSize(((Number)this.lineWidthField.getValue()).floatValue());
        
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_lineWidthFieldActionPerformed
    
    private void bgColorButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bgColorButtonActionPerformed
        java.awt.Color bgColor = javax.swing.JColorChooser.showDialog(this,"title",this.mod.getBackgroundColor());
        if (bgColor != null) {
            this.mod.setBackgroundColor(bgColor);
            this.redisplayParent();
        }
    }//GEN-LAST:event_bgColorButtonActionPerformed

    private void colorSchemeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_colorSchemeButtonActionPerformed
        Coloring c = Coloring.coloringOptionPane(this,this.mod.getColorer());
        if (c != null) {
            this.mod.setColorer(c);
            this.redisplayParent();
        }
    }//GEN-LAST:event_colorSchemeButtonActionPerformed

    private void newSysButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newSysButtonActionPerformed
        ParametricModuleSetupPanel pmsp = new ParametricModuleSetupPanel(this.mod.getSystem());
        net.java.joglutils.JPanelDialog jpd = new net.java.joglutils.JPanelDialog(pmsp);
        if(jpd.showAsModal()) {
            try {
                this.mod.setSystem(pmsp.makeSystem());
                
                this.moduleSettingsToPanel(this.mod);
                
                this.currIterField.setText(Integer.toString(mod.getSystem().getCurrentIteration()));
                
                this.redisplayParent();
                this.repaint();
            } catch (SDDMException e) {
                javax.swing.JOptionPane.showMessageDialog(this,"Problem generating new system:"+e.getMessage(),"Couldn't apply system",javax.swing.JOptionPane.WARNING_MESSAGE);
            }
        }
    }//GEN-LAST:event_newSysButtonActionPerformed

    private void rangeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_rangeButtonActionPerformed
        float x0 = ((Number)this.x0Field.getValue()).floatValue();
        float y0 = ((Number)this.x0Field.getValue()).floatValue();
        float z0 = ((Number)this.x0Field.getValue()).floatValue();
        float w0 = ((Number)this.x0Field.getValue()).floatValue();
        float t0 = ((Number)this.minRangeField.getValue()).floatValue();
        float tf = ((Number)this.maxRangeField.getValue()).floatValue();
        mod.getSystem().generateRange(x0,y0,z0,w0,t0,tf);
        
        this.currIterField.setText(Integer.toString(mod.getSystem().getCurrentIteration()));
        this.currIterField.repaint();
        
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_rangeButtonActionPerformed

    private void iterButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_iterButtonActionPerformed
        int iters = ((Number)this.iterationsField.getValue()).intValue();
        try {
            if (iters == 1)
                mod.getSystem().iterate();
            else
                mod.getSystem().iterate(iters);
        } catch (SDDMException e) {
            javax.swing.JOptionPane.showMessageDialog(null,e.toString(),"Invalid Number",javax.swing.JOptionPane.ERROR_MESSAGE);
        }
        
        currIterField.setText(Integer.toString(mod.getSystem().getCurrentIteration()));
        
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_iterButtonActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bgColorButton;
    private javax.swing.JButton colorSchemeButton;
    private javax.swing.JTextField currIterField;
    private javax.swing.JFormattedTextField granField;
    private javax.swing.JButton iterButton;
    private javax.swing.JCheckBox iterOnReDispCheckBox;
    private javax.swing.JFormattedTextField iterationsField;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JCheckBox joinCheckBox;
    private javax.swing.JFormattedTextField lineWidthField;
    private javax.swing.JFormattedTextField maxRangeField;
    private javax.swing.JFormattedTextField minRangeField;
    private javax.swing.JButton newSysButton;
    private javax.swing.JButton rangeButton;
    private javax.swing.JButton saveButton;
    private javax.swing.JButton targetCenterButton;
    private javax.swing.JFormattedTextField w0Field;
    private javax.swing.JFormattedTextField x0Field;
    private javax.swing.JFormattedTextField y0Field;
    private javax.swing.JFormattedTextField z0Field;
    // End of variables declaration//GEN-END:variables
    
    public void paint(java.awt.Graphics g) {
        super.paint(g);
        this.currIterField.setText(Integer.toString(mod.getSystem().getCurrentIteration()));
    }
}
