/*
 * PointMap3DModule.java
 *
 * Created on August 28, 2006, 12:18 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package sddm.modules;

import sddm.*;
import sddm.data.*;
import javax.media.opengl.*;
import static javax.media.opengl.GL.*;
import com.sun.opengl.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import net.java.joglutils.lighting.*;
/**
 *
 * @author Erik J. Tollerud
 */
public abstract class PointMap3DModule extends SDDModule {
    protected PointList list;
    protected Integer modelList;
    protected GLUT glut;
    protected boolean regenerateModels;
    protected SelectListener selector;
    protected int selectionIndex;
    
    protected float polyOffset;
    protected float shininess;
    protected float lineWidth;
    protected Light light;
    protected Coloring polygonColorer;
    protected Coloring outlineColorer;
    protected boolean lightingOn;
    protected boolean lightOutline;
    protected boolean drawOutline;
    protected boolean drawPolygons;
    protected boolean stipple;
    protected boolean colorByIndex;
    protected Color globalAmbient;
    protected int granularity;
    
    /** Creates a new instance of PointMap3DModule */
    public PointMap3DModule(PointList list) {
        this(list,new StereoCamera());
    }
    
    public PointMap3DModule(PointList list, StereoCamera camera) {
        super(camera);
        this.list = list;
        glut = new GLUT();
        selector = null;
        selectionIndex = -1;
        light = new Light();
        modelList = null;
        polygonColorer = new Coloring();
        outlineColorer = new Coloring(new Color(0.0f,0.0f,0.0f,1.0f));
        
        polyOffset = -4.0f;
        shininess = 100;
        lineWidth = 1.0f;
        granularity = 1;
        globalAmbient = new Color(0.2f,0.2f,0.2f,1.0f);
        lightingOn = true;
        lightOutline = false;
        stipple = false;
        drawPolygons = true;
        drawOutline = false;
        regenerateModels = false;
        colorByIndex = false;
        cam.setCameraLocation(list.getXMax(),list.getYMax(),list.getZMax());
        cam.setTargetPoint(list.getXAvg(),list.getYAvg(),list.getZAvg());
        
    }
    
    public abstract void applyColorBounds(Coloring colorer);
    protected abstract void manipulatePostTranslate(GL gl, int index);
    protected abstract float[] determineColor(Coloring colorer, int index);
    protected abstract Float getSelectionValue();
    public abstract String getSelectionInfoString(java.text.NumberFormat indexnf,java.text.NumberFormat valnf, String delim);
    protected abstract void drawModel(GL gl);
    
    public void init(GL gl) {
        gl.glEnable(GL_NORMALIZE);
        gl.glEnable(GL_DEPTH_TEST);
        light.setLightNumber(0);
        light.initializePhongShader(gl);
        if (lightingOn) {
            try{
                light.enable(gl, 0);
            } catch(LightingException e) {
                JOptionPane.showMessageDialog(null,"Problem setting Phong shader: "+e.getMessage(),"Phong Shading Failed",JOptionPane.WARNING_MESSAGE);
            }
        }
        float[] white = {1.0f,1.0f,1.0f,1.0f};
        gl.glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,white,0);
        generateModelLists(gl);
    }
    
    public void preDraw(GL gl) {
        if(stipple) {
            gl.glEnable(GL_LINE_STIPPLE);
            gl.glLineStipple(3, (short)0xAAAA);
        } else {
            gl.glDisable(GL_LINE_STIPPLE);
        }
        if (regenerateModels) {
            generateModelLists(gl);
            regenerateModels = false;
        }
        
        if (lightingOn) {
            gl.glEnable(GL_NORMALIZE);
            gl.glEnable(GL_LIGHTING);
            
            gl.glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
            gl.glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,this.shininess);
            gl.glEnable(GL_COLOR_MATERIAL);
            Material.applyGlobalAmbient(gl, this.globalAmbient);
            light.apply(gl);
            
            try {
                light.setPhongColorMaterial(true,true);
                light.enable(gl);
            } catch (LightingException e) {
                JOptionPane.showMessageDialog(null,"Problem setting Phong shader: "+e.getMessage(),"Phong Shading Failed",JOptionPane.WARNING_MESSAGE);
            }
        } else {
            gl.glDisable(GL_NORMALIZE);
            gl.glDisable(GL_COLOR_MATERIAL);
            gl.glDisable(GL_LIGHTING);
            light.disable(gl);
        }
        
        gl.glLineWidth(this.lineWidth);
    }
    
    public void drawGeometry(GL gl) {
        gl.glMatrixMode(GL_MODELVIEW);
        
        this.applyColorBounds(polygonColorer);
        this.applyColorBounds(outlineColorer);
        
        for (int i = 0; i < this.list.size(); i+=granularity) {
            gl.glPushMatrix();
            float[] point = this.list.getPoint(i);
            //TODO: do tests to see if this is faster with the branching or with straight division
            if (point[3] == 1.0f)
                gl.glTranslatef(point[0],point[1],point[2]);
            else
                gl.glTranslatef(point[0]/point[3],point[1]/point[3],point[2]/point[3]);
            
            this.manipulatePostTranslate(gl,i);
            
            if(this.drawPolygons) {
                if (this.lightingOn) {
                    light.enable(gl);
                    gl.glEnable(GL_LIGHTING);
                }
                
                gl.glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
                
                float[] color = colorByIndex ? polygonColorer.getColorByDirectValue(i) : determineColor(polygonColorer,i);
                gl.glColor4fv(color,0);
                gl.glPolygonMode(GL.GL_FRONT_AND_BACK,GL.GL_FILL);
                
                gl.glCallList(this.modelList);
            }
            
            if (this.drawOutline) {
                if (!this.lightOutline) {
                    light.disable(gl);
                    gl.glDisable(GL_LIGHTING);
                }
                
                float[] color = colorByIndex ? outlineColorer.getColorByDirectValue(i) : determineColor(outlineColorer,i);
                gl.glColor4fv(color,0);
                gl.glPolygonOffset(polyOffset,1);
                gl.glPolygonMode(GL.GL_FRONT_AND_BACK,GL.GL_LINE);
                
                gl.glCallList(this.modelList);
            }
            gl.glPopMatrix();
        }
    }
    
    public void recompileDisplayList() {
        super.recompileDisplayList();
        if(isSelecting())
            selector.recompile();
        minSizeCache = null;
    }
    
    public void setSelecting(GLAutoDrawable drawable) {
        if (drawable != null) {
            //TODO:clear up the module so that the Pbuffer can be used? (its the display list ints that are the problem)
            //selector = new SelectListener(this,drawable.getWidth(),drawable.getHeight()) {
            selector = new SelectListener(this,drawable) {
                Coloring oldColor;
                boolean dol;
                boolean light;
                boolean cbi;
                
                public void storeOld(SDDModule module) {
                    PointMap3DModule p3d = (PointMap3DModule)module;
                    
                    oldColor = p3d.polygonColorer;
                    dol = p3d.drawOutline;
                    light = p3d.lightingOn;
                    cbi = p3d.colorByIndex;
                    
                }
                
                public void reapplyOld(SDDModule module) {
                    PointMap3DModule p3d = (PointMap3DModule)module;
                    
                    p3d.polygonColorer = oldColor;
                    p3d.drawOutline = dol;
                    p3d.lightingOn = light;
                    p3d.colorByIndex = cbi;
                    
                }
                
                public void setupModForSelect(SDDModule module, Coloring indexedColorer) {
                    PointMap3DModule p3d = (PointMap3DModule)module;
                    
                    p3d.polygonColorer = indexedColorer;
                    p3d.drawOutline = false;
                    p3d.lightingOn = false;
                    p3d.colorByIndex = true;
                }
                
            };
        } else
            selector = null;
    }
    
    public boolean isSelecting() {
        return (selector != null);
    }
    
    Float minSizeCache = null;
    public float getMinFeatureSizeHint() {
        //TODO: do this smarter and cache
        if (list == null)
            return super.getMinFeatureSizeHint();
        
        if (minSizeCache == null) {
            float cellSize = (list.getXMax() - list.getXMin()) * (list.getYMax() - list.getYMin()) * (list.getZMax() - list.getZMin()) / list.size();
            minSizeCache = cellSize /10.0f;
        }
        
        return minSizeCache;
    }
    
    public static String moduleTitle() {
        return "PointMap 3D Drawing Module";
    }
    
    public void attachEventListeners(StereoCameraControlFrame frame) {
        super.attachEventListeners(frame);
        final StereoCameraControlFrame frameF = frame;
        frame.addKeyListenerToJFrames( new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                switch(e.getKeyChar()) {
                    case 'x':
                        if ( isDrawAxis() ) {
                            if (getNumOfTicks() == 0)
                                setDrawAxis(true,list.getXMax(),list.getYMax(),list.getZMax(),5);
                            else
                                setDrawAxis(false,0,0,0);
                        } else
                            setDrawAxis(true,list.getXMax(),list.getYMax(),list.getZMax(),0);
                        frameF.redisplay();
                        break;
                    case 'o':
                        setDrawOutline(!isDrawOutline());
                        recompileDisplayList();
                        frameF.redisplay();
                        frameF.repaint();
                        break;
                    case 'p':
                        setDrawPolygons(!isDrawPolygons());
                        recompileDisplayList();
                        frameF.redisplay();
                        frameF.repaint();
                        break;
                    case 't':
                        setStipple(!isStipple());
                        recompileDisplayList();
                        frameF.redisplay();
                        frameF.repaint();
                        break;
                    case 'L':
                        setLightOutline(!isLightOutline());
                        recompileDisplayList();
                        frameF.redisplay();
                        frameF.repaint();
                        break;
                    case 'l':
                        setLightingOn(!isLightingOn());
                        recompileDisplayList();
                        frameF.redisplay();
                        frameF.repaint();
                        break;
                }
            }
        });
        frame.addMouseListenerToJFrames(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent evt) {
                try {
                    if (isSelecting()) {
                        selectionIndex = selector.selectInt(evt.getX(),evt.getY());
                        frameF.redisplay();
                        frameF.repaint();
                    }
                } catch(InterruptedException e) {
                    throw new SDDMException("Thread interupted while doing selection",e);
                }
            }
            
        });
    }
    
    public int getSelectionIndex() {
        return this.selectionIndex;
    }
    
    public void setLineWidth(float lineWidth) {
        this.lineWidth = lineWidth;
    }
    
    public float getLineWidth() {
        return this.lineWidth;
    }
    
    public void setGranularity(int granularity) {
        this.granularity = granularity;
        this.recompileDisplayList();
    }
    
    public int getGranularity() {
        return this.granularity;
    }
    
    public void setOutlineColoring(Coloring colorer) {
        if (colorer!= null) {
            this.outlineColorer = colorer;
            this.recompileDisplayList();
        }
    }
    
    public Coloring getOutlineColoring() {
        return this.outlineColorer;
    }
    
    public void setPolygonColoring(Coloring colorer) {
        if (colorer!= null) {
            this.polygonColorer = colorer;
            this.recompileDisplayList();
        }
    }
    
    public Coloring getPolygonColoring() {
        return this.polygonColorer;
    }
    
    public void setLightingOn(boolean lighting) {
        this.lightingOn = lighting;
        this.recompileDisplayList();
    }
    
    public boolean isLightingOn() {
        return this.lightingOn;
    }
    
    public void setLightOutline(boolean lightOutline) {
        this.lightOutline = lightOutline;
        this.recompileDisplayList();
    }
    
    public boolean isLightOutline() {
        return this.lightOutline;
    }
    
    public void setColorByIndex(boolean colorByIndex) {
        this.colorByIndex = colorByIndex;
        this.recompileDisplayList();
    }
    
    public boolean isColorByIndex() {
        return this.colorByIndex;
    }
    
    public void setGlobalAmbient(Color globalAmbient) {
        this.recompileDisplayList();
        this.globalAmbient = globalAmbient;
    }
    
    public Color getGlobalAmbient() {
        return globalAmbient;
    }
    
    public void setLight(Light light) {
        if ( light != null ) {
            this.light = light;
            this.recompileDisplayList();
        }
    }
    
    public Light getLight() {
        return this.light;
    }
    
    public void setShininess(float shininess) {
        this.shininess = shininess;
        this.recompileDisplayList();
    }
    
    public float getShininess() {
        return this.shininess;
    }
    
    public void setDrawPolygons(boolean draw) {
        this.drawPolygons = draw;
        this.recompileDisplayList();
    }
    
    public boolean isDrawPolygons() {
        return this.drawPolygons;
    }
    
    public void setDrawOutline(boolean draw) {
        this.drawOutline = draw;
        this.recompileDisplayList();
    }
    
    public boolean isDrawOutline() {
        return this.drawOutline;
    }
    
    public void setStipple(boolean stipple) {
        this.stipple = stipple;
    }
    
    public boolean isStipple() {
        return this.stipple;
    }
    
    public void setPolygonOffset(float polyOffset) {
        this.polyOffset = polyOffset;
        this.recompileDisplayList();
    }
    
    public float getPolygonOffset() {
        return this.polyOffset;
    }
    
    private void generateModelLists(GL gl) {
        if (this.modelList != null)
            gl.glDeleteLists(this.modelList,1);
        this.modelList = gl.glGenLists(1);
        gl.glPolygonMode(GL.GL_FRONT_AND_BACK,GL.GL_FILL);
        gl.glNewList(this.modelList,GL_COMPILE);
        drawModel(gl);
        gl.glEndList();
    }
    
}
