/*
 * S3test.java
 *
 * Created on August 28, 2006, 1:27 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package sddm.modules;

import java.text.*;
import javax.media.opengl.*;
import javax.swing.*;
import sddm.*;
import sddm.data.*;
import static javax.media.opengl.GL.*;

/**
 *
 * @author Erik J. Tollerud
 */
public class Scalar3DModule extends PointMap3DModule {
    public enum Model {SPHERE,CONE,CUBE,TEAPOT,TORUS,DODECAHEDRON,ICOSAHEDRON,OCTAHEDRON,TETRAHEDRON,CUSTOM};
    
    private PointMap<Number> map;
    
    protected Model model;
    protected double[] modelDoubles;
    protected int[] modelInts;
    protected float[][] modelVerticies;
    
    protected boolean zeroBaseScaling;
    protected float globalScale;
    protected float[] logFactors;
    protected float[] expFactors;
    protected float[] polyFactors;

    /** Creates a new instance of S3test */
    public Scalar3DModule() {
        this(new PointMap<Number>(),new StereoCamera());
    }
    
    public Scalar3DModule(StereoCamera camera) {
        this(new PointMap<Number>(),camera);
    }
    
    public Scalar3DModule(PointMap<? extends Number> map) {
        this(map,new StereoCamera());
    }
    
    public Scalar3DModule(PointMap<? extends Number> map, StereoCamera camera) {
        super(map.clone(),camera);
        this.map = (PointMap<Number>)super.list;
        
        zeroBaseScaling = true;
        globalScale = 1.0f;
        modelDoubles = null;
        modelInts = null;
        modelVerticies = null;
        
        this.useSphereModel(8,8);
        this.setGlobalScaleToFillBox(0.5f);
        
    }
    
    protected float[] determineColor(Coloring colorer, int index) {
        return colorer.getColorByValue(this.map.get(index).floatValue());
    }
    
    protected void manipulatePostTranslate(GL gl, int index) {
        float val = this.map.get(index).floatValue();
        float offset = 0.0f;
        
        try {
            if(!this.zeroBaseScaling)
                offset = this.map.getMinValue().floatValue();
        } catch (NoSuchMethodException e) {
            throw new SDDMError("Could not get value from Scalar3DModule",e);
        }
        
        float scaleVal = val-offset;
        
        if (this.logFactors != null) 
            scaleVal = (float)Math.log(logFactors[0]*scaleVal+logFactors[1])+logFactors[2];
        else if (this.expFactors != null)
            scaleVal = (float)Math.exp(expFactors[0]*scaleVal+expFactors[1])+expFactors[2];
        else if (this.polyFactors != null) {
            float x = scaleVal;
            float currPoly = 1;
            for(float coeff : this.polyFactors) {
                scaleVal+=(coeff*currPoly);
                currPoly*=x;
            }
        }
        
        scaleVal *= this.globalScale;
        gl.glScalef(scaleVal,scaleVal,scaleVal);

    }
    
    public String getSelectionInfoString(NumberFormat indexnf, NumberFormat valnf, String delim) {
        if ( this.selectionIndex < 0 || this.selectionIndex >= this.map.size() )
            return "Invalid Selection";
        else {
            float[] pt = this.map.getPoint(this.selectionIndex);
            Number val = this.map.get(this.selectionIndex);
            return "x:"+indexnf.format(pt[0])+" y:"+indexnf.format(pt[1])+" z:"+indexnf.format(pt[2])+" Value:"+valnf.format(val);
        }
    }
    
    protected void drawModel(GL gl) {
        //TODO:check
        switch (this.model) {
            case SPHERE:
                glut.glutSolidSphere(1.0,this.modelInts[0],this.modelInts[1]);
                break;
            case CONE:
                glut.glutSolidCone(this.modelDoubles[0],this.modelDoubles[1],this.modelInts[0],this.modelInts[1]);
                break;
            case CUBE:
                glut.glutSolidCube(1.0f);
                break;
            case TEAPOT:
                glut.glutSolidTeapot(1.0);
                break;
            case TORUS:
                glut.glutSolidTorus(this.modelDoubles[0],this.modelDoubles[1],this.modelInts[0],this.modelInts[1]);
                break;
            case DODECAHEDRON:
                glut.glutSolidDodecahedron();
                break;
            case ICOSAHEDRON:
                glut.glutSolidIcosahedron();
                break;
            case OCTAHEDRON:
                glut.glutSolidOctahedron();
                break;
            case TETRAHEDRON:
                glut.glutSolidTetrahedron();
                break;
            case CUSTOM:
                gl.glBegin(GL_POLYGON);
                for(float[] f : this.modelVerticies)
                    gl.glVertex4fv(f,0);
                gl.glEnd();
                break;
        }
    }
    
    public static SDDModuleSetupPanel newStartupPanel() {
        return new Scalar3DModuleSetupPanel();
    }
    
    public SDDModuleControlPanel newModuleControlPanel() {
        return new Scalar3DModuleControlPanel(this);
    }
    
    protected Float getSelectionValue() {
        try {
            return this.map.get(this.selectionIndex).floatValue();
        } catch (Exception e) {
            return null;
        }
    }
    
    public void applyColorBounds(Coloring colorer) {
        try {
            colorer.setTop(this.map.getMaxValue().floatValue());
            colorer.setBottom(this.map.getMinValue().floatValue());
        } catch (NoSuchMethodException e) {
            throw new SDDMError("Could not get value from Scalar3DModule",e);
        }
    }
    
    public void setGlobalScale(float globalScale) {
        this.globalScale = globalScale;
    }
    
    public void setGlobalScaleToFillBox(float boxSize) {
        try {
            float max = this.map.getMaxValue().floatValue();
            float min = this.map.getMinValue().floatValue();
            
            max = Math.max(Math.abs(max),Math.abs(min));
            this.setGlobalScale(boxSize/max);
            this.setLinearScaling();
        } catch (NoSuchMethodException e) {
            throw new SDDMError("Could not get value from Scalar3DModule",e);
        }
    }
    
    public float getGlobalScale() {
        return this.globalScale;
    }
    
    public void setZeroBaseScaling(boolean zeroBase) {
        this.zeroBaseScaling = zeroBase;
        this.recompileDisplayList();
    }
    
    public boolean isZeroBaseScaling() {
        return this.zeroBaseScaling;
    }
    
    public void setLinearScaling() {
        this.expFactors = null;
        this.logFactors = null;
        this.polyFactors = null;
    }
    
    public boolean isLinearScaling() {
        return (this.expFactors == null && this.logFactors == null && this.polyFactors == null);
    }
    
    public void setLogarithmicScaling() {
        this.setLogarithmicScaling(1,1,0);
    }
    
    public void setLogarithmicScaling(float gain, float inputOffset,float outputOffset) {
        this.logFactors = new float[3];
        this.logFactors[0] = gain;
        this.logFactors[1] = inputOffset;
        this.logFactors[2] = outputOffset;
        this.expFactors = null;
        this.polyFactors = null;
    }
    
    public boolean isLogarithmicScaling() {
        return this.logFactors != null;
    }
    
    public void setExponentialScaling() {
        this.setExponentialScaling(1,0,-1);
    }
    
    public void setExponentialScaling(float gain, float inputOffset,float outputOffset) {
        this.expFactors = new float[3];
        this.expFactors[0] = gain;
        this.expFactors[1] = inputOffset;
        this.expFactors[2] = outputOffset;
        this.logFactors = null;
        this.polyFactors = null;
    }
    
    public boolean isExponentialScaling() {
        return this.expFactors != null;
    }
    
    /**
     * Scales from a polynomial function
     * @param coefficients the coefficient of a term of the polynomial, starting at 0th order
     */
    public void setPolynomialScaling(float[] coefficients) {
        this.polyFactors = coefficients;
        this.logFactors = null;
        this.expFactors = null;
    }
    
    public void setPolynomialScaling(float quadratic) {
        float[] coeffs = {0,0,quadratic};
        this.polyFactors = coeffs;
        this.logFactors = null;
        this.expFactors = null;
    }
    
    public void setPolynomialScaling(float quadratic, float cubic) {
        float[] coeffs = {0,0,quadratic,cubic};
        this.polyFactors = coeffs;
        this.logFactors = null;
        this.expFactors = null;
    }
    
    public boolean isPolynomialScaling() {
        return this.polyFactors != null;
    }
    
    public HeightField extractHeightField(javax.vecmath.Vector3f planeNormal, javax.vecmath.Vector4f pointInPlane,int xRes, int yRes) {
        //TODO:implement
        return null;
    }
    
    public void useSphereModel(int slices, int stacks) {
        this.model = Model.SPHERE;
        this.modelDoubles = null;
        this.modelInts = new int[2];
        modelInts[0] = slices;
        modelInts[1] = stacks;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    public void useConeModel(double baseToHeightRatio, int slices, int stacks) {
        this.model = Model.CONE;
        this.modelDoubles = new double[2];
        this.modelInts = new int[2];
        if (baseToHeightRatio > 1.0) {
            modelDoubles[0] = 1.0;//base
            modelDoubles[1] = 1.0/baseToHeightRatio;//height
        } else {
            modelDoubles[0] = baseToHeightRatio;//base
            modelDoubles[1] = 1.0;//height
        }
        modelInts[0] = slices;
        modelInts[1] = stacks;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    /**
     * Uses a torus to represent the points in question
     * @param innerToOuterRadiusRatio should be less than 1
     * @param nsides the number of sides to use in generating the torus' polygons
     * @param rings the number of annular rings to use in generating the torus' polygons
     */
    public void useTorusModel(double innerToOuterRadiusRatio, int nsides, int rings) {
        this.model = Model.TORUS;
        this.modelDoubles = new double[2];
        this.modelInts = new int[2];
        modelDoubles[0] = 1.0 - innerToOuterRadiusRatio;//innerRadius
        modelDoubles[1] = 1.0;//OuterRadius
        modelInts[0] = nsides;
        modelInts[1] = rings;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    public void useCubeModel() {
        this.model = Model.CUBE;
        this.modelDoubles = null;
        this.modelInts = null;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    public void useTeapotModel() {
        this.model = Model.TEAPOT;
        this.modelDoubles = null;
        this.modelInts = null;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    public void useDodecahedronModel() {
        this.model = Model.DODECAHEDRON;
        this.modelDoubles = null;
        this.modelInts = null;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    public void useIcosahedronModel() {
        this.model = Model.ICOSAHEDRON;
        this.modelDoubles = null;
        this.modelInts = null;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    public void useOctahedronModel() {
        this.model = Model.OCTAHEDRON;
        this.modelDoubles = null;
        this.modelInts = null;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    public void useTetrahedronModel() {
        this.model = Model.TETRAHEDRON;
        this.modelDoubles = null;
        this.modelInts = null;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    /**
     *
     * @param verticies a list of 3 or 4-element points
     * @throws sddm.SDDMException if the points are not 3 or 4 elements long
     */
    public void useCustomModel(float[][] verticies) throws SDDMException{
        //TODO:test
        this.model = Model.CUSTOM;
        this.modelDoubles = null;
        this.modelInts = null;
        this.modelVerticies = new float[verticies.length][4];
        for (int i = 0; i < verticies.length; ++i) {
            float[] f = verticies[i];
            if (f.length == 3) {
                this.modelVerticies[i][0] = verticies[i][0];
                this.modelVerticies[i][1] = verticies[i][1];
                this.modelVerticies[i][2] = verticies[i][2];
                this.modelVerticies[i][3] = 1.0f;
            } else if (f.length == 4) {
                this.modelVerticies[i][0] = verticies[i][0];
                this.modelVerticies[i][1] = verticies[i][1];
                this.modelVerticies[i][2] = verticies[i][2];
                this.modelVerticies[i][3] = verticies[i][3];
            } else
                throw new SDDMException("Supplied vertex is not 3 or 4 elements long.");
        }
        
        this.modelDoubles = null;
        this.modelInts = null;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    public float[][] getCustomModel() {
        //TODO: test if this is deep or shallow
        return this.modelVerticies.clone();
    }
    
    public Model getModelType() {
        return this.model;
    }
    
    void setMap(PointMap<Number> newMap) {
        this.map = newMap;
        super.list = newMap;
    }
    
    PointMap<Number> getMap() {
        return this.map;
    }
    
    public static String moduleTitle() { 
        return "Scalar 3D Drawing Module";
    }
    
    public static SDDModule makeModuleFromArgs(String modName, java.util.HashMap<String,Integer> switchMap, String[] args, StereoCamera camera) throws  ParsingException{
        //takes modName s3m or scalar3dmodule
        //-(f|file) file [four] [field]
        //-[g|gran] #
        //
        if (modName.equals("s3m") || modName.equals("scalar3dmodule") ) {
            Integer fileIndx = switchMap.remove("-f");
            if( fileIndx == null )
                fileIndx = switchMap.remove("-file");
            if ( fileIndx == null)
                throw new ParsingException("No data file provided for Scalar3DModule");
            
            java.io.File f = new java.io.File(args[fileIndx+1]);
            boolean threeElement = true;
            boolean field = false;
            if((args.length > fileIndx+2) && args[fileIndx+2].equals("four"))
                threeElement = false;
            if((args.length > fileIndx+2) && args[fileIndx+2].equals("field"))
                field = true;
            if((args.length > fileIndx+3) && args[fileIndx+3].equals("four"))
                threeElement = false;
            if((args.length > fileIndx+3) && args[fileIndx+3].equals("field"))
                field = true;
            
            Scalar3DModule s3m;
            try {
                s3m = new Scalar3DModule(PointMap.loadPointsAndFloatsFromFile(f,threeElement,field),camera);
            } catch (FileFormatException e) {
                JOptionPane.showMessageDialog(null,"File is not corect format:"+e.getMessage()+" Using empty PointMap","File Incorrect Format",JOptionPane.ERROR_MESSAGE);
                s3m = new Scalar3DModule(camera);
            } catch (java.io.FileNotFoundException e) {
                JOptionPane.showMessageDialog(null,"No File With Name "+f+", using empty PointMap","File Not Found",JOptionPane.ERROR_MESSAGE);
                s3m = new Scalar3DModule(camera);
            }
            
            Integer granIndex = switchMap.remove("-g");
            if (granIndex == null)
                granIndex = switchMap.remove("-gran");
            if (granIndex != null) {
                String granStr = null;
                try {
                    granStr = args[granIndex+1];
                    s3m.setGranularity(Integer.getInteger(granStr));
                } catch (Exception e) {
                    throw new ParsingException("Couldn't understand granularity command "+granStr+":"+e.getMessage(),e);
                }
                //TODO:implement more command line options
            }
            
            return s3m;
            
            
        }
        return null;
    }
    
}







///*
// * Scalar3DModule.java
// *
// * Created on August 23, 2006, 11:53 AM
// *
// * To change this template, choose Tools | Template Manager
// * and open the template in the editor.
// */
//
//package sddm.modules;
//
//import sddm.*;
//import sddm.data.*;
//import javax.media.opengl.*;
//import static javax.media.opengl.GL.*;
//import javax.swing.*;
//import java.awt.*;
//import java.awt.event.*;
//import com.sun.opengl.util.*;
//import net.java.joglutils.lighting.*;
//
///**
// *
// * @author Erik J. Tollerud
// * Sample command line: -debug -aa 4 -mod s3m -file "C:\Documents and Settings\Erik\Desktop\module test data\ScalarFields\vcoscharge31.txt" field
// */
//public class Scalar3DModule extends SDDModule{
//    public enum Model {SPHERE,CONE,CUBE,TEAPOT,TORUS,DODECAHEDRON,ICOSAHEDRON,OCTAHEDRON,TETRAHEDRON,CUSTOM};
//    
//    protected Integer modelList;
//    protected PointMap<Number> map;
//    protected GLUT glut;
//    protected Model model;
//    protected double[] modelDoubles;
//    protected int[] modelInts;
//    protected float[][] modelVerticies;
//    protected boolean regenerateModels;
//    protected SelectListener selector;
//    protected int selectionIndex;
//    
//    protected float offset;
//    protected float globalScale;
//    protected float shininess;
//    protected float lineWidth;
//    protected Light light;
//    protected Coloring polygonColorer;
//    protected Coloring outlineColorer;
//    protected boolean lightingOn;
//    protected boolean lightOutline;
//    protected boolean drawOutline;
//    protected boolean drawPolygons;
//    protected boolean stipple;
//    protected boolean zeroBaseScaling;
//    protected boolean colorByIndex;
//    protected Color globalAmbient;
//    protected int granularity;
//    
//    
//    
//    /** Creates a new instance of Scalar3DModule */
//    public Scalar3DModule(StereoCamera camera) {
//        super(camera);
//        map = new PointMap<Number>();
//        glut = new GLUT();
//        modelList = null;
//        modelDoubles = null;
//        modelInts = null;
//        modelVerticies = null;
//        selector = null;
//        selectionIndex = -1;
//        light = new Light();
//        polygonColorer = new Coloring();
//        outlineColorer = new Coloring(new Color(0.0f,0.0f,0.0f,1.0f));
//        globalScale = 1.0f;
//        offset = -4.0f;
//        shininess = 100;
//        lineWidth = 1.0f;
//        granularity = 1;
//        globalAmbient = new Color(0.2f,0.2f,0.2f,1.0f);
//        lightingOn = true;
//        lightOutline = false;
//        stipple = false;
//        drawPolygons = true;
//        drawOutline = false;
//        regenerateModels = false;
//        zeroBaseScaling = true;
//        colorByIndex = false;
//        
//        this.useSphereModel(8,8);
//        
//    }
//    
//    public Scalar3DModule() {
//        this(new StereoCamera());
//    }
//    
//    public Scalar3DModule(PointMap<? extends Number> map) {
//        this(new StereoCamera());
//        this.map = map.clone();
//        setGlobalScaleToFillBox(0.5f);
//        cam.setCameraLocation(2*map.getXMax(),2*map.getYMax(),2*map.getZMax());
//        cam.setTargetPoint(0,0,0);
//    }
//    
//    public Scalar3DModule(PointMap<? extends Number> map, StereoCamera camera) {
//        this(camera);
//        this.map = map.clone();
//        setGlobalScaleToFillBox(0.5f);
//        cam.setCameraLocation(2*map.getXMax(),2*map.getYMax(),2*map.getZMax());
//        cam.setTargetPoint(0,0,0);
//    }
//    
//    public void init(GL gl) {
//        gl.glEnable(GL_NORMALIZE);
//        gl.glEnable(GL_DEPTH_TEST);
//        light.setLightNumber(0);
//        light.initializePhongShader(gl);
//        if (lightingOn) {
//            try{
//                light.enable(gl, 0);
//            } catch(LightingException e) {
//                JOptionPane.showMessageDialog(null,"Problem setting Phong shader: "+e.getMessage(),"Phong Shading Failed",JOptionPane.WARNING_MESSAGE);
//            }
//        }
//        float[] white = {1.0f,1.0f,1.0f,1.0f};
//        gl.glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,white,0);
//        generateModelLists(gl);
//    }
//    
//    public void preDraw(GL gl) {
//        
//        if(stipple) {
//            gl.glEnable(GL_LINE_STIPPLE);
//            gl.glLineStipple(3, (short)0xAAAA);
//        } else {
//            gl.glDisable(GL_LINE_STIPPLE);
//        }
//        if (regenerateModels) {
//            generateModelLists(gl);
//            regenerateModels = false;
//        }
//        
//        if (lightingOn) {
//            gl.glEnable(GL_NORMALIZE);
//            gl.glEnable(GL_LIGHTING);
//            
//            gl.glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
//            gl.glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,this.shininess);
//            gl.glEnable(GL_COLOR_MATERIAL);
//            Material.applyGlobalAmbient(gl, this.globalAmbient);
//            light.apply(gl);
//            
//            try {
//                light.setPhongColorMaterial(true,true);
//                light.enable(gl);
//            } catch (LightingException e) {
//                JOptionPane.showMessageDialog(null,"Problem setting Phong shader: "+e.getMessage(),"Phong Shading Failed",JOptionPane.WARNING_MESSAGE);
//            }
//        } else {
//            gl.glDisable(GL_NORMALIZE);
//            gl.glDisable(GL_COLOR_MATERIAL);
//            gl.glDisable(GL_LIGHTING);
//            light.disable(gl);
//        }
//        
//        gl.glLineWidth(this.lineWidth);
//    }
//    
//    public void drawGeometry(GL gl) {
//        gl.glMatrixMode(GL_MODELVIEW);
//        float offset = 0.0f;
//        
//        try {
//            float min = this.map.getMinValue().floatValue();
//            float max = this.map.getMaxValue().floatValue();
//            this.polygonColorer.setTop(max);
//            this.polygonColorer.setBottom(min);
//            this.outlineColorer.setTop(max);
//            this.outlineColorer.setBottom(min);
//            if(!this.zeroBaseScaling)
//                offset = this.map.getMinValue().floatValue();
//        } catch (NoSuchMethodException e) {
//            throw new SDDMException("Serious problem; Scalar3DModule PointMap does not have elements implementing Comparable.",e);
//        }
//        
//        for (int i = 0; i < this.map.size(); i+=granularity) {
//            gl.glPushMatrix();
//            float val = this.map.get(i).floatValue();
//            float[] point = this.map.getPoint(i);
//            //TODO: do tests to see if this is faster with the branching or with straight division
//            if (point[3] == 1.0f)
//                gl.glTranslatef(point[0],point[1],point[2]);
//            else
//                gl.glTranslatef(point[0]/point[3],point[1]/point[3],point[2]/point[3]);
//            
//            float scaleVal = this.globalScale*(val-offset);
//            gl.glScalef(scaleVal,scaleVal,scaleVal);
//            
//            
//            if(this.drawPolygons) {
//                if (this.lightingOn) {
//                    light.enable(gl);
//                    gl.glEnable(GL_LIGHTING);
//                }
//                
//                gl.glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
//                
//                float[] color = colorByIndex ? polygonColorer.getColorByDirectValue(i) : polygonColorer.getColorByValue(this.map.get(i).floatValue());
//                gl.glColor4fv(color,0);
//                gl.glPolygonMode(GL.GL_FRONT_AND_BACK,GL.GL_FILL);
//                gl.glCallList(this.modelList+1);//solid
//            }
//            
//            if (this.drawOutline) {
//                if (!this.lightOutline) {
//                    light.disable(gl);
//                    gl.glDisable(GL_LIGHTING);
//                }
//                
//                float[] color = colorByIndex ? outlineColorer.getColorByDirectValue(i) : outlineColorer.getColorByValue(this.map.get(i).floatValue());
//                gl.glColor4fv(color,0);
//                gl.glPolygonOffset(offset,1);
//                gl.glPolygonMode(GL.GL_FRONT_AND_BACK,GL.GL_LINE);
//                gl.glCallList(this.modelList);//wire
//            }
//            gl.glPopMatrix();
//        }
//    }
//    
//    public void recompileDisplayList() {
//        super.recompileDisplayList();
//        if(isSelecting())
//            selector.recompile();
//    }
//    
//    public void setSelecting(GLAutoDrawable drawable) {
//        if (drawable != null) {
//            GLDrawableFactory factory = GLDrawableFactory.getFactory();
//            if(!factory.canCreateGLPbuffer())
//                throw new SDDMException("Graphics device does not support Pbuffers - can't enter selection mode");
//            
//            selector = new SelectListener(this,drawable.getWidth(),drawable.getHeight()) {
//                Coloring oldColor;
//                boolean dol;
//                boolean light;
//                boolean cbi;
//                
//                public void storeOld(SDDModule module) {
//                    Scalar3DModule s3m = (Scalar3DModule)module;
//                    
//                    oldColor = s3m.getPolygonColoring();
//                    dol = s3m.drawOutline;
//                    light = s3m.lightingOn;
//                    cbi = s3m.colorByIndex;
//                    
//                }
//                public void reapplyOld(SDDModule module) {
//                    Scalar3DModule s3m = (Scalar3DModule)module;
//                    
//                    s3m.polygonColorer = oldColor;
//                    s3m.drawOutline = dol;
//                    s3m.lightingOn = light;
//                    s3m.colorByIndex = cbi;
//                    
//                }
//                public void setupModForSelect(SDDModule module, Coloring indexedColorer) {
//                    Scalar3DModule s3m = (Scalar3DModule)module;
//                    
//                    s3m.polygonColorer = indexedColorer;
//                    s3m.drawOutline = false;
//                    s3m.lightingOn = false;
//                    s3m.colorByIndex = true;
//                }
//                
//            };
//        } else
//            selector = null;
//    }
//    
//    public boolean isSelecting() {
//        return (selector != null);
//    }
//    
//    public float getMinFeatureSizeHint() {
//        //TODO: do this smarter
//        if (map == null)
//            return super.getMinFeatureSizeHint();
//        float retVal = (map.getXMax() - map.getXMin()) * (map.getYMax() - map.getYMin()) * (map.getZMax() - map.getZMin());
//        return retVal / map.size();
//    }
//    
//    public SDDModuleControlPanel newModuleControlPanel() {
//        return new Scalar3DModuleControlPanel(this);
//    }
//    
//    public static SDDModuleSetupPanel newStartupPanel() {
//        return new Scalar3DModuleSetupPanel();
//    }
//    
//    public static SDDModule makeModuleFromArgs(String modName, java.util.HashMap<String,Integer> switchMap, String[] args, StereoCamera camera) throws  ParsingException{
//        //takes modName s3m or scalar3dmodule
//        //-(f|file) file [four] [field]
//        //-[g|gran] #
//        //
//        if (modName.equals("s3m") || modName.equals("scalar3dmodule") ) {
//            Integer fileIndx = switchMap.remove("-f");
//            if( fileIndx == null )
//                fileIndx = switchMap.remove("-file");
//            if ( fileIndx == null)
//                throw new ParsingException("No data file provided for Scalar3DModule");
//            
//            java.io.File f = new java.io.File(args[fileIndx+1]);
//            boolean threeElement = true;
//            boolean field = false;
//            if((args.length > fileIndx+2) && args[fileIndx+2].equals("four"))
//                threeElement = false;
//            if((args.length > fileIndx+2) && args[fileIndx+2].equals("field"))
//                field = true;
//            if((args.length > fileIndx+3) && args[fileIndx+3].equals("four"))
//                threeElement = false;
//            if((args.length > fileIndx+3) && args[fileIndx+3].equals("field"))
//                field = true;
//            
//            Scalar3DModule s3m;
//            try {
//                s3m = new Scalar3DModule(PointMap.loadPointsAndFloatsFromFile(f,threeElement,field),camera);
//            } catch (FileFormatException e) {
//                JOptionPane.showMessageDialog(null,"File is not corect format:"+e.getMessage()+" Using empty PointMap","File Incorrect Format",JOptionPane.ERROR_MESSAGE);
//                s3m = new Scalar3DModule(camera);
//            } catch (java.io.FileNotFoundException e) {
//                JOptionPane.showMessageDialog(null,"No File With Name "+f+", using empty PointMap","File Not Found",JOptionPane.ERROR_MESSAGE);
//                s3m = new Scalar3DModule(camera);
//            }
//            
//            Integer granIndex = switchMap.remove("-g");
//            if (granIndex == null)
//                granIndex = switchMap.remove("-gran");
//            if (granIndex != null) {
//                String granStr = null;
//                try {
//                    granStr = args[granIndex+1];
//                    s3m.setGranularity(Integer.getInteger(granStr));
//                } catch (Exception e) {
//                    throw new ParsingException("Couldn't understand granularity command "+granStr+":"+e.getMessage(),e);
//                }
//                //TODO:implement more command line options
//            }
//            
//            return s3m;
//            
//            
//        }
//        return null;
//    }
//    
//    public static String moduleTitle() {
//        return "Scalar 3D Drawing Module";
//    }
//    
//    public void attachEventListeners(StereoCameraControlFrame frame) {
//        super.attachEventListeners(frame);
//        final StereoCameraControlFrame frameF = frame;
//        frame.addKeyListenerToJFrames( new java.awt.event.KeyAdapter() {
//            public void keyPressed(KeyEvent e) {
//                switch(e.getKeyChar()) {
//                    case 'x':
//                        if ( isDrawAxis() ) {
//                            if (getNumOfTicks() == 0)
//                                setDrawAxis(true,map.getXMax(),map.getYMax(),map.getZMax(),5);
//                            else
//                                setDrawAxis(false,0,0,0);
//                        } else
//                            setDrawAxis(true,map.getXMax(),map.getYMax(),map.getZMax(),0);
//                        frameF.redisplay();
//                        break;
//                    case 'o':
//                        setDrawOutline(!isDrawOutline());
//                        recompileDisplayList();
//                        frameF.redisplay();
//                        break;
//                    case 'p':
//                        setDrawPolygons(!isDrawPolygons());
//                        recompileDisplayList();
//                        frameF.redisplay();
//                        break;
//                    case 't':
//                        setStipple(!isStipple());
//                        recompileDisplayList();
//                        frameF.redisplay();
//                        break;
//                    case 'L':
//                        setLightOutline(!isLightOutline());
//                        recompileDisplayList();
//                        frameF.redisplay();
//                        break;
//                    case 'l':
//                        setLightingOn(!isLightingOn());
//                        recompileDisplayList();
//                        frameF.redisplay();
//                        break;
//                }
//            }
//            
//        });
//        frame.addMouseListenerToJFrames(new java.awt.event.MouseAdapter() {
//            public void mouseClicked(MouseEvent evt) {
//                try {
//                    if (isSelecting()) {
//                        selectionIndex = selector.selectInt(evt.getX(),evt.getY());
//                        frameF.redisplay();
//                    }
//                } catch(InterruptedException e) {
//                    throw new SDDMException("Thread interupted while doing selection",e);
//                }
//            }
//            
//        });
//    }
//    
//    public int getSelectionIndex() {
//        return this.selectionIndex;
//    }
//    
//    public Float getSelectionValue() {
//        try {
//            return this.map.get(this.selectionIndex).floatValue();
//        }
//        catch (Exception e) {
//            return null;
//        }
//    }
//    
//    /**
//     * Reports an informational string based on the selected point. 15 Characters are used between returned numbers.
//     * 
//     * @return a string describing the selected point
//     * @param delim the delimiter to use between the elements of the description
//     * @param indexnf the number-formatter to use for reporting the locations of the point
//     * @param valnf the number-formatter to use for reporting the value at the point
//     */
//    public String getSelectionInfoString(java.text.NumberFormat indexnf,java.text.NumberFormat valnf, String delim) {
//        if ( this.selectionIndex < 0 || this.selectionIndex >= this.map.size() )
//            return "Invalid Selection";
//        else {
//            float[] pt = this.map.getPoint(this.selectionIndex);
//            Number val = this.map.get(this.selectionIndex);
//            return "x:"+indexnf.format(pt[0])+" y:"+indexnf.format(pt[1])+" z:"+indexnf.format(pt[2])+" Value:"+valnf.format(val);
//        }
//    }
//
//    public void setLineWidth(float lineWidth) {
//        this.lineWidth = lineWidth;
//    }
//    
//    public float getLineWidth() {
//        return this.lineWidth;
//    }
//    
//    public void setGranularity(int granularity) {
//        this.granularity = granularity;
//        this.recompileDisplayList();
//    }
//    
//    public int getGranularity() {
//        return this.granularity;
//    }
//    
//    public void setZeroBaseScaling(boolean zeroBase) {
//        this.zeroBaseScaling = zeroBase;
//        this.recompileDisplayList();
//    }
//    
//    public boolean isZeroBaseScaling() {
//        return this.zeroBaseScaling;
//    }
//    
//    public void setGlobalScale(float globalScale) {
//        this.globalScale = globalScale;
//    }
//    
//    public void setGlobalScaleToFillBox(float boxSize) {
//        float max;
//        float min;
//        try {
//            max = this.map.getMaxValue().floatValue();
//            min = this.map.getMinValue().floatValue();
//        } catch (NoSuchMethodException e) {
//            throw new SDDMException("Serious problem - Scalar 3D Module does not contain comparable objects...",e);
//        }
//        
//        max = Math.max(Math.abs(max),Math.abs(min));
//        this.setGlobalScale(boxSize/max);
//    }
//    
//    public float getGlobalScale() {
//        return this.globalScale;
//    }
//    
//    public void setOutlineColoring(Coloring colorer) {
//        if (colorer!= null) {
//            this.outlineColorer = colorer;
//            this.recompileDisplayList();
//        }
//    }
//    
//    public Coloring getOutlineColoring() {
//        return this.outlineColorer;
//    }
//    
//    public void setPolygonColoring(Coloring colorer) {
//        if (colorer!= null) {
//            this.polygonColorer = colorer;
//            this.recompileDisplayList();
//        }
//    }
//    
//    public Coloring getPolygonColoring() {
//        return this.polygonColorer;
//    }
//    
//    public void setLightingOn(boolean lighting) {
//        this.lightingOn = lighting;
//        this.recompileDisplayList();
//    }
//    
//    public boolean isLightingOn() {
//        return this.lightingOn;
//    }
//    
//    public void setLightOutline(boolean lightOutline) {
//        this.lightOutline = lightOutline;
//        this.recompileDisplayList();
//    }
//    
//    public boolean isLightOutline() {
//        return this.lightOutline;
//    }
//    
//    public void setColorByIndex(boolean colorByIndex) {
//        this.colorByIndex = colorByIndex;
//        this.recompileDisplayList();
//    }
//    
//    public boolean isColorByIndex() {
//        return this.colorByIndex;
//    }
//    
//    public void setGlobalAmbient(Color globalAmbient) {
//        this.recompileDisplayList();
//        this.globalAmbient = globalAmbient;
//    }
//    
//    public Color getGlobalAmbient() {
//        return globalAmbient;
//    }
//    
//    public void setLight(Light light) {
//        if ( light != null ) {
//            this.light = light;
//            this.recompileDisplayList();
//        }
//    }
//    
//    public Light getLight() {
//        return this.light;
//    }
//    
//    public void setShininess(float shininess) {
//        this.shininess = shininess;
//        this.recompileDisplayList();
//    }
//    
//    public float getShininess() {
//        return this.shininess;
//    }
//    
//    public void setDrawPolygons(boolean draw) {
//        this.drawPolygons = draw;
//        this.recompileDisplayList();
//    }
//    
//    public boolean isDrawPolygons() {
//        return this.drawPolygons;
//    }
//    
//    public void setDrawOutline(boolean draw) {
//        this.drawOutline = draw;
//        this.recompileDisplayList();
//    }
//    
//    public boolean isDrawOutline() {
//        return this.drawOutline;
//    }
//    
//    public void setStipple(boolean stipple) {
//        this.stipple = stipple;
//    }
//    
//    public boolean isStipple() {
//        return this.stipple;
//    }
//    
//    public void setPolygonOffset(float offset) {
//        this.offset = offset;
//        this.recompileDisplayList();
//    }
//    
//    public float getPolygonOffset() {
//        return this.offset;
//    }
//    
//    public void useSphereModel(int slices, int stacks) {
//        this.model = Model.SPHERE;
//        this.modelDoubles = null;
//        this.modelInts = new int[2];
//        modelInts[0] = slices;
//        modelInts[1] = stacks;
//        this.regenerateModels = true;
//        this.recompileDisplayList();
//    }
//    
//    public void useConeModel(double baseToHeightRatio, int slices, int stacks) {
//        this.model = Model.CONE;
//        this.modelDoubles = new double[2];
//        this.modelInts = new int[2];
//        if (baseToHeightRatio > 1.0) {
//            modelDoubles[0] = 1.0;//base
//            modelDoubles[1] = 1.0/baseToHeightRatio;//height
//        } else {
//            modelDoubles[0] = baseToHeightRatio;//base
//            modelDoubles[1] = 1.0;//height
//        }
//        modelInts[0] = slices;
//        modelInts[1] = stacks;
//        this.regenerateModels = true;
//        this.recompileDisplayList();
//    }
//    
//    /**
//     * Uses a torus to represent the points in question
//     * @param innerToOuterRadiusRatio should be less than 1
//     * @param nsides the number of sides to use in generating the torus' polygons
//     * @param rings the number of annular rings to use in generating the torus' polygons
//     */
//    public void useTorusModel(double innerToOuterRadiusRatio, int nsides, int rings) {
//        this.model = Model.TORUS;
//        this.modelDoubles = new double[2];
//        this.modelInts = new int[2];
//        modelDoubles[0] = 1.0/innerToOuterRadiusRatio;//innerRadius
//        modelDoubles[1] = 1.0;//OuterRadius
//        modelInts[0] = nsides;
//        modelInts[1] = rings;
//        this.regenerateModels = true;
//        this.recompileDisplayList();
//    }
//    
//    public void useCubeModel() {
//        this.model = Model.CUBE;
//        this.modelDoubles = null;
//        this.modelInts = null;
//        this.regenerateModels = true;
//        this.recompileDisplayList();
//    }
//    
//    public void useTeapotModel() {
//        this.model = Model.TEAPOT;
//        this.modelDoubles = null;
//        this.modelInts = null;
//        this.regenerateModels = true;
//        this.recompileDisplayList();
//    }
//    
//    public void useDodecahedronModel() {
//        this.model = Model.DODECAHEDRON;
//        this.modelDoubles = null;
//        this.modelInts = null;
//        this.regenerateModels = true;
//        this.recompileDisplayList();
//    }
//    
//    public void useIcosahedronModel() {
//        this.model = Model.ICOSAHEDRON;
//        this.modelDoubles = null;
//        this.modelInts = null;
//        this.regenerateModels = true;
//        this.recompileDisplayList();
//    }
//    
//    public void useOctahedronModel() {
//        this.model = Model.OCTAHEDRON;
//        this.modelDoubles = null;
//        this.modelInts = null;
//        this.regenerateModels = true;
//        this.recompileDisplayList();
//    }
//    
//    public void useTetrahedronModel() {
//        this.model = Model.TETRAHEDRON;
//        this.modelDoubles = null;
//        this.modelInts = null;
//        this.regenerateModels = true;
//        this.recompileDisplayList();
//    }
//    
//    /**
//     *
//     * @param verticies a list of 3 or 4-element points
//     * @throws sddm.SDDMException if the points are not 3 or 4 elements long
//     */
//    public void useCustomModel(float[][] verticies) throws SDDMException{
//        //TODO:test
//        this.model = Model.CUSTOM;
//        this.modelDoubles = null;
//        this.modelInts = null;
//        this.modelVerticies = new float[verticies.length][4];
//        for (int i = 0; i < verticies.length; ++i) {
//            float[] f = verticies[i];
//            if (f.length == 3) {
//                this.modelVerticies[i][0] = verticies[i][0];
//                this.modelVerticies[i][1] = verticies[i][1];
//                this.modelVerticies[i][2] = verticies[i][2];
//                this.modelVerticies[i][3] = 1.0f;
//            } else if (f.length == 4) {
//                this.modelVerticies[i][0] = verticies[i][0];
//                this.modelVerticies[i][1] = verticies[i][1];
//                this.modelVerticies[i][2] = verticies[i][2];
//                this.modelVerticies[i][3] = verticies[i][3];
//            } else
//                throw new SDDMException("Supplied vertex is not 3 or 4 elements long.");
//        }
//        
//        this.modelDoubles = null;
//        this.modelInts = null;
//        this.regenerateModels = true;
//        this.recompileDisplayList();
//    }
//    
//    public float[][] getCustomModel() {
//        //TODO: test if this is deep or shallow
//        return this.modelVerticies.clone();
//    }
//    
//    public Model getModelType() {
//        return this.model;
//    }
//    
//    //--------------------------privates below------------------------------
//    //two lists are generated: first is wire, second is solid
//    private void generateModelLists(GL gl ) {
//        if (this.modelList != null)
//            gl.glDeleteLists(this.modelList,2);
//        this.modelList = gl.glGenLists(2);
//        gl.glNewList(this.modelList,GL_COMPILE);
//        switch (this.model) {
//            case SPHERE:
//                glut.glutWireSphere(1.0,this.modelInts[0],this.modelInts[1]);
//                gl.glEndList();
//                gl.glNewList(this.modelList+1,GL_COMPILE);
//                glut.glutSolidSphere(1.0,this.modelInts[0],this.modelInts[1]);
//                break;
//            case CONE:
//                glut.glutWireCone(this.modelDoubles[0],this.modelDoubles[1],this.modelInts[0],this.modelInts[1]);
//                gl.glEndList();
//                gl.glNewList(this.modelList+1,GL_COMPILE);
//                glut.glutSolidCone(this.modelDoubles[0],this.modelDoubles[1],this.modelInts[0],this.modelInts[1]);
//                break;
//            case CUBE:
//                glut.glutWireCube(1.0f);
//                gl.glEndList();
//                gl.glNewList(this.modelList+1,GL_COMPILE);
//                glut.glutSolidCube(1.0f);
//                break;
//            case TEAPOT:
//                glut.glutWireTeapot(1.0);
//                gl.glEndList();
//                gl.glNewList(this.modelList+1,GL_COMPILE);
//                glut.glutSolidTeapot(1.0);
//                break;
//            case TORUS:
//                glut.glutWireTorus(this.modelDoubles[0],this.modelDoubles[1],this.modelInts[0],this.modelInts[1]);
//                gl.glEndList();
//                gl.glNewList(this.modelList+1,GL_COMPILE);
//                glut.glutSolidTorus(this.modelDoubles[0],this.modelDoubles[1],this.modelInts[0],this.modelInts[1]);
//                break;
//            case DODECAHEDRON:
//                glut.glutWireDodecahedron();
//                gl.glEndList();
//                gl.glNewList(this.modelList+1,GL_COMPILE);
//                glut.glutSolidDodecahedron();
//                break;
//            case ICOSAHEDRON:
//                glut.glutWireIcosahedron();
//                gl.glEndList();
//                gl.glNewList(this.modelList+1,GL_COMPILE);
//                glut.glutSolidIcosahedron();
//                break;
//            case OCTAHEDRON:
//                glut.glutWireOctahedron();
//                gl.glEndList();
//                gl.glNewList(this.modelList+1,GL_COMPILE);
//                glut.glutSolidOctahedron();
//                break;
//            case TETRAHEDRON:
//                glut.glutWireTetrahedron();
//                gl.glEndList();
//                gl.glNewList(this.modelList+1,GL_COMPILE);
//                glut.glutSolidTetrahedron();
//                break;
//            case CUSTOM:
//                gl.glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
//                generateCustomPolygon(gl);
//                gl.glEndList();
//                gl.glNewList(this.modelList+1,GL_COMPILE);
//                gl.glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
//                generateCustomPolygon(gl);
//                break;
//        }
//        gl.glEndList();
//    }
//    
//    private void generateCustomPolygon(GL gl) {
//        gl.glBegin(GL_POLYGON);
//        for(float[] f : this.modelVerticies)
//            gl.glVertex4fv(f,0);
//        gl.glEnd();
//    }
//}
