/*
 * TestModule.java
 *
 * Created on June 12, 2006, 3:30 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package sddm.modules;
import javax.media.opengl.*;
import java.awt.*;
import sddm.*;
import sddm.SDDModule;
import sddm.SDDModuleControlPanel;

/**
 *
 * @author Erik J. Tollerud
 */
public class TestModule extends SDDModule {
    Color renderColor;
    boolean showText;
    int comboBoxIndex;
    float lineWidth;
    boolean displayInfo;
    boolean useMatrix;
    javax.vecmath.Matrix4f mat;
    boolean tile;
    // 1 means an 3X3X3 pattern, 2 gives 5X5X5, 3 results in 7X7X7, etc.
    int radialUnitCells;
    float tileScale;
    
    /** Creates a new instance of TestModule */
    public TestModule() {
        this(new StereoCamera());
        
    }
    public TestModule(StereoCamera camera) {
        super(camera);
        cam.setCameraLocation(2.0f,2.0f,2.0f);
        cam.setTargetPoint(0,0,0);
        StereoCamera.FrustumSettings frs = cam.getFrustumSettings();
        frs.cameraFocalLength = 2.5f;
        cam.setFrustumSettings(frs);
        showText = true;
        displayInfo = true;
        renderColor = new Color(0.45f,0.2f,0.631f);
        comboBoxIndex = 0;
        lineWidth = 2;
        useMatrix = false;
        mat = new javax.vecmath.Matrix4f(1.0f,0.0f,0.0f,0.0f,0.0f,1.0f,0.0f,0.0f,0.0f,0.0f,1.0f,0.0f,0.0f,0.0f,0.0f,1.0f);
        tile = false;
        tileScale = 1.0f;
        radialUnitCells = 1;
    }
    
    public float[] getMatrixAsArray(){
        float[] result = new float[16];
        result[0] = mat.m00;
        result[1] = mat.m10;
        result[2] = mat.m20;
        result[3] = mat.m30;
        result[4] = mat.m01;
        result[5] = mat.m11;
        result[6] = mat.m21;
        result[7] = mat.m31;
        result[8] = mat.m02;
        result[9] = mat.m12;
        result[10] = mat.m22;
        result[11] = mat.m32;
        result[12] = mat.m03;
        result[13] = mat.m13;
        result[14] = mat.m23;
        result[15] = mat.m33;
        return result;
    }
    
    public void init(GL g) {
        //float[] bgc = bgColor.getRGBColorComponents(null);
        //g.glClearColor(bgc[0],bgc[1],bgc[2],0.0f);
        //turns out that geting the integers and setting individual values is slightly better performance-wise
        g.glEnable(g.GL_DEPTH_TEST);
        g.glDisable(g.GL_CULL_FACE);
        g.glCullFace(g.GL_BACK);
        
    }
    
    public void drawGeometry(GL g) {
        if (useMatrix)
            g.glMultMatrixf(getMatrixAsArray(),0);
        
        if (displayInfo) {
            System.err.println("Drawing Geometry");
            javax.vecmath.Vector3f c = cam.getCameraLocation();
            javax.vecmath.Vector3f t = cam.getTargetLocation();
            javax.vecmath.Vector3f u = cam.getUpVector();
            javax.vecmath.Vector3f n = cam.getViewDirection();
            System.err.println("view:"+n+" p:"+u);
            System.err.println("camloc:"+c+" targloc:"+t);
        }
        
        g.glLineWidth(this.lineWidth);
        
        //float[] bgc = bgColor.getRGBColorComponents(null);
        //g.glClearColor(bgc[0],bgc[1],bgc[2],0.0f);
        //turns out that geting the integers and setting individual values is slightly better performance-wise
        g.glClear(g.GL_COLOR_BUFFER_BIT|g.GL_DEPTH_BUFFER_BIT);
        
        com.sun.opengl.util.GLUT glut = new com.sun.opengl.util.GLUT();
        
        //Draw object at origin
        //float[] rc = renderColor.getRGBColorComponents(null);
        //g.glClearColor(rc[0],rc[1],rc[2],0.0f);
        //turns out that geting the integers and setting individual values is slightly better performance-wise
        g.glColor3ub((byte)renderColor.getRed(),(byte)renderColor.getGreen(),(byte)renderColor.getBlue());
        
        if (tile) {
            int r = this.radialUnitCells;
            //g.glScalef(this.tileScale,this.tileScale,this.tileScale);
            for(int i = -r ; i <= r; i++) {
                for(int j = -r ; j <= r; j++) {
                    for(int k = -r ; k <= r; k++) {
                        g.glPushMatrix();
                        g.glTranslatef(i,j,k);
                        g.glScalef(this.tileScale,this.tileScale,this.tileScale);
                        renderCentralObject(glut);
                        g.glPopMatrix();

                    }
                }
            }
        } else
            renderCentralObject(glut);

        if(showText) {
            g.glPushAttrib(g.GL_ENABLE_BIT);
            g.glDisable(g.GL_DEPTH_TEST);
            g.glMatrixMode(g.GL_PROJECTION);
            g.glPushMatrix();
            g.glLoadIdentity();
            g.glMatrixMode(g.GL_MODELVIEW);
            g.glPushMatrix();
            g.glLoadIdentity();
            g.glRasterPos2d(0.4,0.8);
            glut.glutBitmapString(glut.BITMAP_HELVETICA_18,"Test module");
            g.glPopMatrix();
            g.glMatrixMode(g.GL_PROJECTION);
            g.glPopMatrix();
            g.glMatrixMode(g.GL_MODELVIEW);
            g.glPopAttrib();
        }
        
        
        //Draw X-axis in red
        g.glColor3d(1.0,0,0);
        g.glBegin(g.GL_LINES);
        g.glVertex3i(0,0,0);
        g.glVertex3i(1,0,0);
        g.glEnd();
        //Draw Y-axis in green
        g.glColor3d(0,1.0,0);
        g.glBegin(g.GL_LINES);
        g.glVertex3i(0,0,0);
        g.glVertex3i(0,1,0);
        g.glEnd();
        //Draw Z-axis in blue
        g.glColor3d(0,0,1.0);
        g.glBegin(g.GL_LINES);
        g.glVertex3i(0,0,0);
        g.glVertex3i(0,0,1);
        g.glEnd();
        
        
    }

    private void renderCentralObject(final com.sun.opengl.util.GLUT glut) {
            
        switch (comboBoxIndex) {
            case 1:
                glut.glutWireSphere(1.0,25,10);
                break;
            case 2:
                glut.glutWireCube(1.0f);
                break;
            case 3:
                glut.glutWireCone(1.0,1.0,25,10);
                break;
            case 4:
                glut.glutWireDodecahedron();
                break;
            case 5:
                glut.glutWireIcosahedron();
                break;
            case 6:
                glut.glutWireOctahedron();
                break;
            case 7:
                glut.glutWireTetrahedron();
                break;
            case 0:
            default:
                glut.glutWireTeapot(1);
        }
    }
    public static String moduleTitle() {
        return "Test Module";
    }
    
    public SDDModuleControlPanel newModuleControlPanel() {
        SDDModuleControlPanel smcp = new TestModulePanel(this);
        return smcp;
    }
    
    public void recompile() {
        this.recompileDisplayList();
    }

    public float getMinFeatureSizeHint() {
        return 0.05f;
    }
    
    public static SDDModule makeModuleFromArgs(String modName, java.util.HashMap<String,Integer> switchMap, String[] args, StereoCamera camera) throws  ParsingException{
        if (modName.equalsIgnoreCase("test")) {
            //switches accepted
            //-shape teapot|sphere|cube|cone|dodecahedron|icosahedron|octahedron|tetrahedron
            //-linesize #
            //-bgcolor r g b
            //-rendercolor r g b
            
            TestModule m = new TestModule(camera);
            
            Integer shapeIndx = switchMap.remove("-shape");
            if(shapeIndx != null) {
                String shapeStr = args[shapeIndx+1];
                if (shapeStr.equalsIgnoreCase("teapot")) {
                    m.comboBoxIndex = 0;
                }
                else if (shapeStr.equalsIgnoreCase("sphere")) {
                    m.comboBoxIndex = 1;
                }
                else if (shapeStr.equalsIgnoreCase("cube")) {
                    m.comboBoxIndex = 2;
                }
                else if (shapeStr.equalsIgnoreCase("cone")) {
                    m.comboBoxIndex = 3;
                }
                else if (shapeStr.equalsIgnoreCase("dodecahedron")) {
                    m.comboBoxIndex = 4;
                }
                else if (shapeStr.equalsIgnoreCase("icosahedron")) {
                    m.comboBoxIndex = 5;
                }
                else if (shapeStr.equalsIgnoreCase("octahedron")) {
                    m.comboBoxIndex = 6;
                }
                else if (shapeStr.equalsIgnoreCase("tetrahedron")) {
                    m.comboBoxIndex = 7;
                }
                else 
                    throw new ParsingException("unrecognized shape in test module:"+shapeStr);
                
            }
            Integer lineIndx = switchMap.remove("-linesize");
            if(lineIndx != null) {
                float lineSize = Float.parseFloat(args[lineIndx+1]);
                m.lineWidth = lineSize;
            }
            Integer bgIndx = switchMap.remove("-bgcolor");
            if(bgIndx != null) {
                float r = Float.parseFloat(args[bgIndx+1]);
                float g = Float.parseFloat(args[bgIndx+2]);
                float b = Float.parseFloat(args[bgIndx+3]);
                m.setBackgroundColor(new Color(r,g,b));
            }
            Integer renderIndx = switchMap.remove("-rendercolor");
            if(renderIndx != null) {
                float r = Float.parseFloat(args[renderIndx+1]);
                float g = Float.parseFloat(args[renderIndx+2]);
                float b = Float.parseFloat(args[renderIndx+3]);
                m.renderColor = new Color(r,g,b);
            }
            
            return m;
        }
        return null;
    }
}
